﻿using System;
using System.Collections;
using System.Collections.ObjectModel;
using System.Net;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows;
using Telerik.Windows.Controls.ScheduleView;
using Telerik.Windows.Controls;

namespace ScheduleViewUpdateHeaders
{
    public class AvailableTimeHelper : DependencyObject
    {
        public static readonly DependencyProperty HelperProperty =
             DependencyProperty.RegisterAttached("Helper", typeof(AvailableTimeHelper), typeof(AvailableTimeHelper), null);

        public static readonly DependencyProperty AppointmentsProperty =
            DependencyProperty.Register("Appointments", typeof(NotifyItemPropertyChangedCollection<Appointment>), typeof(AvailableTimeHelper), new System.Windows.PropertyMetadata(OnAppointmentsPropertyChanged));

        public static readonly DependencyProperty DateProperty =
            DependencyProperty.Register("Date", typeof(object), typeof(AvailableTimeHelper), new System.Windows.PropertyMetadata(OnDatePropertyChanged));

        public static readonly DependencyProperty TextProperty =
            DependencyProperty.Register("Text", typeof(string), typeof(AvailableTimeHelper), null);

        public static readonly DependencyProperty ResourceProperty =
            DependencyProperty.Register("Resource", typeof(object), typeof(AvailableTimeHelper), new System.Windows.PropertyMetadata(OnDatePropertyChanged));

        public NotifyItemPropertyChangedCollection<Appointment> Appointments
        {
            get { return (NotifyItemPropertyChangedCollection<Appointment>)GetValue(AppointmentsProperty); }
            set { SetValue(AppointmentsProperty, value); }
        }

        public object Date
        {
            get { return (object)GetValue(DateProperty); }
            set { SetValue(DateProperty, value); }
        }

        public string Text
        {
            get { return (string)GetValue(TextProperty); }
            set { SetValue(TextProperty, value); }
        }

        public object Resource
        {
            get { return (object)GetValue(ResourceProperty); }
            set { SetValue(ResourceProperty, value); }
        }

        public static AvailableTimeHelper GetHelper(DependencyObject obj)
        {
            return (AvailableTimeHelper)obj.GetValue(HelperProperty);
        }

        public static void SetHelper(DependencyObject obj, AvailableTimeHelper value)
        {
            obj.SetValue(HelperProperty, value);
        }

        private static void OnDatePropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var appointmentsCounterHelper = (AvailableTimeHelper)d;
            appointmentsCounterHelper.Update();
        }

        private static void OnAppointmentsPropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var appointmentsCounterHelper = (AvailableTimeHelper)d;
            var oldCollection = (NotifyItemPropertyChangedCollection<Appointment>)e.OldValue;
            var newCollection = (NotifyItemPropertyChangedCollection<Appointment>)e.NewValue;

            if (e.OldValue != null)
            {
                oldCollection.ItemPropertyChanged -= appointmentsCounterHelper.AppointmentsPropertyChangedCollection_ItemPropertyChanged;
                newCollection.CollectionChanged -= appointmentsCounterHelper.NewCollection_CollectionChanged;
                oldCollection = null;
            }

            if (e.NewValue != null)
            {
                newCollection.ItemPropertyChanged += appointmentsCounterHelper.AppointmentsPropertyChangedCollection_ItemPropertyChanged;
                newCollection.CollectionChanged += appointmentsCounterHelper.NewCollection_CollectionChanged;
            }

            appointmentsCounterHelper.Update();
        }

        private void NewCollection_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            this.Update();
        }

        private void AppointmentsPropertyChangedCollection_ItemPropertyChanged(object sender, ItemPropertyChangedEventArgs<Appointment> e)
        {
            this.Update();
        }

        private void Update()
        {
            if (this.Date != null && this.Resource != null)
            {
                var resName = (Resource)(this.Resource as IGroupItemInfo).Name;
                var date = (DateTime) this.Date;

                int result = this.Appointments.Where(x => x.RecurrenceRule == null && x.Resources.Contains(resName)).Count(x => x.IntersectsWith(new DateSpan(date.AddMinutes(1), date.AddDays(1).AddMinutes(-1))));
                result += this.Appointments.Where(x => x.RecurrenceRule != null && x.Resources.Contains(resName)).Where(r => r.GetOccurrences(date.AddMinutes(1), date.AddDays(1).AddMinutes(-1)).Count > 0).Count();
               
                this.Text = "Appointments for " + resName + " :" + result;
            }
        }
    }
}
