﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;

namespace ScheduleViewUpdateHeaders
{
    public class NotifyItemPropertyChangedCollection<T> : ObservableCollection<T>
    {
        public NotifyItemPropertyChangedCollection()
            : base()
        {
        }

        public NotifyItemPropertyChangedCollection(IEnumerable<T> collection)
            : base(collection)
        {
            foreach (var item in collection.OfType<INotifyPropertyChanged>())
            {
                item.PropertyChanged += this.OnItemPropertyChanged;
            }
        }

        public event EventHandler<ItemPropertyChangedEventArgs<T>> ItemPropertyChanged;

        protected override void ClearItems()
        {
            foreach (var item in this.OfType<INotifyPropertyChanged>())
            {
                item.PropertyChanged -= this.OnItemPropertyChanged;
            }

            base.ClearItems();
        }

        protected override void InsertItem(int index, T item)
        {
            var propertyChangedItem = item as INotifyPropertyChanged;
            if (propertyChangedItem != null)
            {
                propertyChangedItem.PropertyChanged += this.OnItemPropertyChanged;
            }

            base.InsertItem(index, item);
        }

        protected override void RemoveItem(int index)
        {
            var propertyChangedItem = this[index] as INotifyPropertyChanged;
            if (propertyChangedItem != null)
            {
                propertyChangedItem.PropertyChanged -= this.OnItemPropertyChanged;
            }

            base.RemoveItem(index);
        }

        protected override void SetItem(int index, T item)
        {
            var propertyChangedItem = this[index] as INotifyPropertyChanged;
            if (propertyChangedItem != null)
            {
                propertyChangedItem.PropertyChanged -= this.OnItemPropertyChanged;
            }

            propertyChangedItem = item as INotifyPropertyChanged;
            if (propertyChangedItem != null)
            {
                propertyChangedItem.PropertyChanged += this.OnItemPropertyChanged;
            }

            base.SetItem(index, item);
        }

        protected virtual void OnItemPropertyChanged(ItemPropertyChangedEventArgs<T> e)
        {
            if (this.ItemPropertyChanged != null)
            {
                this.ItemPropertyChanged(this, e);
            }
        }

        private void OnItemPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            this.OnItemPropertyChanged(new ItemPropertyChangedEventArgs<T>((T)sender, e.PropertyName));
        }
    }
}
